# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from . import invoice_pb2 as invoice_dot_grpc_dot_invoice__pb2


class CreateInvoiceControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Create = self.channel.unary_unary(
                "/config.invoice.CreateInvoiceController/Create",
                request_serializer=invoice_dot_grpc_dot_invoice__pb2.CreateInvoiceRequest.SerializeToString,
                response_deserializer=invoice_dot_grpc_dot_invoice__pb2.CreateInvoiceResponse.FromString,
            )


class CreateInvoiceControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CreateInvoiceControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Create": grpc.unary_unary_rpc_method_handler(
            servicer.Create,
            request_deserializer=invoice_dot_grpc_dot_invoice__pb2.CreateInvoiceRequest.FromString,
            response_serializer=invoice_dot_grpc_dot_invoice__pb2.CreateInvoiceResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.invoice.CreateInvoiceController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CreateInvoiceController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.invoice.CreateInvoiceController/Create",
            invoice_dot_grpc_dot_invoice__pb2.CreateInvoiceRequest.SerializeToString,
            invoice_dot_grpc_dot_invoice__pb2.CreateInvoiceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class InquiryInvoiceControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Retrieve = self.channel.unary_unary(
                "/config.invoice.InquiryInvoiceController/Retrieve",
                request_serializer=invoice_dot_grpc_dot_invoice__pb2.InquiryInvoiceRetrieveRequest.SerializeToString,
                response_deserializer=invoice_dot_grpc_dot_invoice__pb2.InquiryInvoiceResponse.FromString,
            )


class InquiryInvoiceControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_InquiryInvoiceControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=invoice_dot_grpc_dot_invoice__pb2.InquiryInvoiceRetrieveRequest.FromString,
            response_serializer=invoice_dot_grpc_dot_invoice__pb2.InquiryInvoiceResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.invoice.InquiryInvoiceController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class InquiryInvoiceController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.invoice.InquiryInvoiceController/Retrieve",
            invoice_dot_grpc_dot_invoice__pb2.InquiryInvoiceRetrieveRequest.SerializeToString,
            invoice_dot_grpc_dot_invoice__pb2.InquiryInvoiceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class InquiryPaymentControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Retrieve = self.channel.unary_unary(
                "/config.invoice.InquiryPaymentController/Retrieve",
                request_serializer=invoice_dot_grpc_dot_invoice__pb2.InquiryPaymentRetrieveRequest.SerializeToString,
                response_deserializer=invoice_dot_grpc_dot_invoice__pb2.InquiryPaymentResponse.FromString,
            )


class InquiryPaymentControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_InquiryPaymentControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=invoice_dot_grpc_dot_invoice__pb2.InquiryPaymentRetrieveRequest.FromString,
            response_serializer=invoice_dot_grpc_dot_invoice__pb2.InquiryPaymentResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.invoice.InquiryPaymentController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class InquiryPaymentController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.invoice.InquiryPaymentController/Retrieve",
            invoice_dot_grpc_dot_invoice__pb2.InquiryPaymentRetrieveRequest.SerializeToString,
            invoice_dot_grpc_dot_invoice__pb2.InquiryPaymentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class PaymentInvoiceControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.PaymentRequest = self.channel.unary_unary(
                "/config.invoice.PaymentInvoiceController/PaymentRequest",
                request_serializer=invoice_dot_grpc_dot_invoice__pb2.PaymentRequestRequest.SerializeToString,
                response_deserializer=invoice_dot_grpc_dot_invoice__pb2.PaymentRequestResponse.FromString,
            )
            self.VerifyPayment = self.channel.unary_unary(
                "/config.invoice.PaymentInvoiceController/VerifyPayment",
                request_serializer=invoice_dot_grpc_dot_invoice__pb2.VerifyPaymentRequest.SerializeToString,
                response_deserializer=invoice_dot_grpc_dot_invoice__pb2.VerifyPaymentResponse.FromString,
            )


class PaymentInvoiceControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def PaymentRequest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def VerifyPayment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_PaymentInvoiceControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "PaymentRequest": grpc.unary_unary_rpc_method_handler(
            servicer.PaymentRequest,
            request_deserializer=invoice_dot_grpc_dot_invoice__pb2.PaymentRequestRequest.FromString,
            response_serializer=invoice_dot_grpc_dot_invoice__pb2.PaymentRequestResponse.SerializeToString,
        ),
        "VerifyPayment": grpc.unary_unary_rpc_method_handler(
            servicer.VerifyPayment,
            request_deserializer=invoice_dot_grpc_dot_invoice__pb2.VerifyPaymentRequest.FromString,
            response_serializer=invoice_dot_grpc_dot_invoice__pb2.VerifyPaymentResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.invoice.PaymentInvoiceController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class PaymentInvoiceController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def PaymentRequest(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.invoice.PaymentInvoiceController/PaymentRequest",
            invoice_dot_grpc_dot_invoice__pb2.PaymentRequestRequest.SerializeToString,
            invoice_dot_grpc_dot_invoice__pb2.PaymentRequestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def VerifyPayment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.invoice.PaymentInvoiceController/VerifyPayment",
            invoice_dot_grpc_dot_invoice__pb2.VerifyPaymentRequest.SerializeToString,
            invoice_dot_grpc_dot_invoice__pb2.VerifyPaymentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class UserListInvoiceControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.UserRelatedList = self.channel.unary_unary(
                "/config.invoice.UserListInvoiceController/UserRelatedList",
                request_serializer=invoice_dot_grpc_dot_invoice__pb2.UserListInvoiceRequest.SerializeToString,
                response_deserializer=invoice_dot_grpc_dot_invoice__pb2.UserListInvoiceResponse.FromString,
            )


class UserListInvoiceControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def UserRelatedList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_UserListInvoiceControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "UserRelatedList": grpc.unary_unary_rpc_method_handler(
            servicer.UserRelatedList,
            request_deserializer=invoice_dot_grpc_dot_invoice__pb2.UserListInvoiceRequest.FromString,
            response_serializer=invoice_dot_grpc_dot_invoice__pb2.UserListInvoiceResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.invoice.UserListInvoiceController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class UserListInvoiceController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def UserRelatedList(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.invoice.UserListInvoiceController/UserRelatedList",
            invoice_dot_grpc_dot_invoice__pb2.UserListInvoiceRequest.SerializeToString,
            invoice_dot_grpc_dot_invoice__pb2.UserListInvoiceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: invoice/grpc/invoice.proto
# Protobuf Python Version: 4.25.1
# pylint: skip-file
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1ainvoice/grpc/invoice.proto\x12\x0e\x63onfig.invoice\x1a\x1cgoogle/protobuf/struct.proto"\x93\x02\n\x14\x43reateInvoiceRequest\x12\x18\n\x10owner_identifier\x18\x01 \x01(\t\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x05\x12\x30\n\nextra_data\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x12\x1e\n\x11\x62\x61sket_identifier\x18\x05 \x01(\tH\x01\x88\x01\x01\x12\x1c\n\x0f\x64iscount_amount\x18\x06 \x01(\x05H\x02\x88\x01\x01\x12\x18\n\x0b\x64\x65scription\x18\x07 \x01(\tH\x03\x88\x01\x01\x42\r\n\x0b_extra_dataB\x14\n\x12_basket_identifierB\x12\n\x10_discount_amountB\x0e\n\x0c_description"]\n\x15\x43reateInvoiceResponse\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x0c\n\x04\x63ode\x18\x04 \x01(\t\x12%\n\x04\x64\x61ta\x18\x05 \x03(\x0b\x32\x17.google.protobuf.Struct"^\n\x16InquiryInvoiceResponse\x12\x0f\n\x07message\x18\x07 \x01(\t\x12\x0c\n\x04\x63ode\x18\x08 \x01(\t\x12%\n\x04\x64\x61ta\x18\t \x03(\x0b\x32\x17.google.protobuf.Struct"6\n\x1dInquiryInvoiceRetrieveRequest\x12\x15\n\rtracking_code\x18\x01 \x01(\t"^\n\x16InquiryPaymentResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"6\n\x1dInquiryPaymentRetrieveRequest\x12\x15\n\rtracking_code\x18\x01 \x01(\t"c\n\x15PaymentRequestRequest\x12\x18\n\x0b\x64\x65scription\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0f\n\x07gateway\x18\x02 \x01(\t\x12\x0f\n\x07invoice\x18\x03 \x01(\tB\x0e\n\x0c_description"^\n\x16PaymentRequestResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"2\n\x16UserListInvoiceRequest\x12\x18\n\x10owner_identifier\x18\x01 \x01(\t"_\n\x17UserListInvoiceResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"J\n\x14VerifyPaymentRequest\x12\x16\n\x0egateway_status\x18\x01 \x01(\t\x12\x1a\n\x12payment_identifier\x18\x02 \x01(\t"]\n\x15VerifyPaymentResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct2r\n\x17\x43reateInvoiceController\x12W\n\x06\x43reate\x12$.config.invoice.CreateInvoiceRequest\x1a%.config.invoice.CreateInvoiceResponse"\x00\x32\x7f\n\x18InquiryInvoiceController\x12\x63\n\x08Retrieve\x12-.config.invoice.InquiryInvoiceRetrieveRequest\x1a&.config.invoice.InquiryInvoiceResponse"\x00\x32\x7f\n\x18InquiryPaymentController\x12\x63\n\x08Retrieve\x12-.config.invoice.InquiryPaymentRetrieveRequest\x1a&.config.invoice.InquiryPaymentResponse"\x00\x32\xdd\x01\n\x18PaymentInvoiceController\x12\x61\n\x0ePaymentRequest\x12%.config.invoice.PaymentRequestRequest\x1a&.config.invoice.PaymentRequestResponse"\x00\x12^\n\rVerifyPayment\x12$.config.invoice.VerifyPaymentRequest\x1a%.config.invoice.VerifyPaymentResponse"\x00\x32\x81\x01\n\x19UserListInvoiceController\x12\x64\n\x0fUserRelatedList\x12&.config.invoice.UserListInvoiceRequest\x1a\'.config.invoice.UserListInvoiceResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "invoice.grpc.invoice_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_CREATEINVOICEREQUEST"]._serialized_start = 77
    _globals["_CREATEINVOICEREQUEST"]._serialized_end = 352
    _globals["_CREATEINVOICERESPONSE"]._serialized_start = 354
    _globals["_CREATEINVOICERESPONSE"]._serialized_end = 447
    _globals["_INQUIRYINVOICERESPONSE"]._serialized_start = 449
    _globals["_INQUIRYINVOICERESPONSE"]._serialized_end = 543
    _globals["_INQUIRYINVOICERETRIEVEREQUEST"]._serialized_start = 545
    _globals["_INQUIRYINVOICERETRIEVEREQUEST"]._serialized_end = 599
    _globals["_INQUIRYPAYMENTRESPONSE"]._serialized_start = 601
    _globals["_INQUIRYPAYMENTRESPONSE"]._serialized_end = 695
    _globals["_INQUIRYPAYMENTRETRIEVEREQUEST"]._serialized_start = 697
    _globals["_INQUIRYPAYMENTRETRIEVEREQUEST"]._serialized_end = 751
    _globals["_PAYMENTREQUESTREQUEST"]._serialized_start = 753
    _globals["_PAYMENTREQUESTREQUEST"]._serialized_end = 852
    _globals["_PAYMENTREQUESTRESPONSE"]._serialized_start = 854
    _globals["_PAYMENTREQUESTRESPONSE"]._serialized_end = 948
    _globals["_USERLISTINVOICEREQUEST"]._serialized_start = 950
    _globals["_USERLISTINVOICEREQUEST"]._serialized_end = 1000
    _globals["_USERLISTINVOICERESPONSE"]._serialized_start = 1002
    _globals["_USERLISTINVOICERESPONSE"]._serialized_end = 1097
    _globals["_VERIFYPAYMENTREQUEST"]._serialized_start = 1099
    _globals["_VERIFYPAYMENTREQUEST"]._serialized_end = 1173
    _globals["_VERIFYPAYMENTRESPONSE"]._serialized_start = 1175
    _globals["_VERIFYPAYMENTRESPONSE"]._serialized_end = 1268
    _globals["_CREATEINVOICECONTROLLER"]._serialized_start = 1270
    _globals["_CREATEINVOICECONTROLLER"]._serialized_end = 1384
    _globals["_INQUIRYINVOICECONTROLLER"]._serialized_start = 1386
    _globals["_INQUIRYINVOICECONTROLLER"]._serialized_end = 1513
    _globals["_INQUIRYPAYMENTCONTROLLER"]._serialized_start = 1515
    _globals["_INQUIRYPAYMENTCONTROLLER"]._serialized_end = 1642
    _globals["_PAYMENTINVOICECONTROLLER"]._serialized_start = 1645
    _globals["_PAYMENTINVOICECONTROLLER"]._serialized_end = 1866
    _globals["_USERLISTINVOICECONTROLLER"]._serialized_start = 1869
    _globals["_USERLISTINVOICECONTROLLER"]._serialized_end = 1998
# @@protoc_insertion_point(module_scope)



from .grpc_lib.invoice_pb2 import (
    CreateInvoiceRequest,
    InquiryInvoiceRetrieveRequest,
    InquiryPaymentRetrieveRequest,
    PaymentRequestRequest,
    UserListInvoiceRequest,
    VerifyPaymentRequest,
)
from .grpc_lib.invoice_pb2_grpc import (
    CreateInvoiceControllerStub,
    InquiryInvoiceControllerStub,
    InquiryPaymentControllerStub,
    PaymentInvoiceControllerStub,
    UserListInvoiceControllerStub,
)

SUCCESS_CODE = "0"


class InvoiceRequestMessages:
    CREATE = CreateInvoiceRequest()
    INQUIRY = InquiryInvoiceRetrieveRequest()
    FETCH_USER_INVOICES = UserListInvoiceRequest()


class PaymentRequestMessages:
    REQUEST = PaymentRequestRequest()
    VERIFY = VerifyPaymentRequest()
    INQUIRY = InquiryPaymentRetrieveRequest()


class InvoiceRPCMethodNames:
    CREATE = "Create"
    INQUIRY = "Retrieve"
    FETCH_USER_INSTANCES = "UserRelatedList"


class PaymentRPCMethodNames:
    REQUEST = "PaymentRequest"
    VERIFY = "VerifyPayment"
    INQUIRY = "Retrieve"


class InvoiceControllerStubs:
    CREATE = CreateInvoiceControllerStub()
    INQUIRY = InquiryInvoiceControllerStub()
    FETCH_USER_INVOICES = UserListInvoiceControllerStub()


class PaymentControllerStubs:
    INVOICE = PaymentInvoiceControllerStub()
    INQUIRY = InquiryPaymentControllerStub()



from typing import ClassVar

from google.protobuf.json_format import MessageToDict
from grpc.aio import insecure_channel
from pydantic import UUID4, BaseModel, validate_call

from .api_constants import (
    SUCCESS_CODE,
    InvoiceControllerStubs,
    InvoiceRequestMessages,
    InvoiceRPCMethodNames,
    PaymentControllerStubs,
    PaymentRequestMessages,
    PaymentRPCMethodNames,
)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class Base(BaseModel):
    """
    Configure the host and port of the search server.

    Parameters:
        host (str): The host address of the search server.
        port (int): The port number of the search server.
    """

    host: str = "127.0.0.1"
    port: str | int = "50051"
    api_key: str

    __instance: ClassVar = None

    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    async def perform_operation(
        self, rpc_method_name, request_message, controller_stub
    ):
        """Perform the operation.

        Parameters:
            rpc_method_name (str): The name of the RPC method (e.g., `Create`).
            request_message (gRPC message): The request message instance.
        """
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            controller_stub.channel = channel
            controller_stub.ready()
            RPCMethod = getattr(controller_stub, rpc_method_name, None)

            if RPCMethod:
                metadata = [("api-key", self.api_key)]
                response = await RPCMethod(request_message, metadata=metadata)

                # Clear the request message for future use
                request_message.Clear()

                # Raise error if response is not successful
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)

                return MessageToDict(response, preserving_proto_field_name=True)
            else:
                raise ValidationError("Invalid RPC method name.")

    @validate_call
    async def inquiry(self, tracking_code: str | UUID4):
        """
        Inquiry an invoice/payment using `tracking_code`.

        Args:
            tracking_code (str): The tracking code of the invoice/payment.

        Return:
            Essentialy the invoice/payment info from the response from the server.
        """
        # Assign the tracking code to the request message
        self.inquiry_request.tracking_code = tracking_code

        # Perform the Inquiry operation
        return await self.perform_operation(
            self.inquiry_method_name,  # RPC method name
            self.inquiry_request,  # Request message
            self.inquiry_controller_stub,  # Controller stub
        )


class Invoice(Base):
    inquiry_request: ClassVar = InvoiceRequestMessages.INQUIRY
    inquiry_method_name: ClassVar = InvoiceRPCMethodNames.INQUIRY
    inquiry_controller_stub: ClassVar = InvoiceControllerStubs.INQUIRY

    @validate_call
    async def create(
        self,
        owner_identifier: str | UUID4,
        amount: int,
        discount_amount: int | None = None,
        description: str | None = None,
        extra_data: dict | None = None,
        basket_identifier: str | UUID4 = None,
    ):
        """
        Create an invoice.

        Args:
            owner_identifier: The identifier of the owner.
            amount: The amount of the invoice.
            discount_amount: The discount_amount of the invoice.
            description: The description about the invoice.
            extra_data: The extra data of the invoice.
            basket_identifier: The basket_identifier invoice is being created for.

        Return:
            Message containing success or the error code.
        """
        # Assign attributes to the CREATE request message
        InvoiceRequestMessages.CREATE.owner_identifier = str(owner_identifier)
        InvoiceRequestMessages.CREATE.amount = amount
        if extra_data:
            InvoiceRequestMessages.CREATE.extra_data = extra_data
        if basket_identifier:
            InvoiceRequestMessages.CREATE.basket_identifier = str(basket_identifier)
        if discount_amount:
            InvoiceRequestMessages.CREATE.discount_amount = discount_amount
        if description:
            InvoiceRequestMessages.CREATE.description = description

        # Perform Create operation
        # The `perform_operation` method handles the RPC call
        return await self.perform_operation(
            InvoiceRPCMethodNames.CREATE,  # The RPC method name
            InvoiceRequestMessages.CREATE,  # The request message
            InvoiceControllerStubs.CREATE,  # The controller stub
        )

    @validate_call
    async def fetch_user_invoices(self, owner_identifier: str | UUID4):
        """
        List the invoices of a user.

        Return:
            Essentialy the user invoices list info from the response from the server.
        """
        # Assign the tracking code to the request message
        InvoiceRequestMessages.FETCH_USER_INVOICES.owner_identifier = owner_identifier

        # Perform the Inquiry operation
        return await self.perform_operation(
            InvoiceRPCMethodNames.FETCH_USER_INSTANCES,  # RPC method name
            InvoiceRequestMessages.FETCH_USER_INVOICES,  # Request message
            InvoiceControllerStubs.FETCH_USER_INVOICES,  # Controller stub
        )

    # These two methods moved to Payment class but also kept here for backward compatibility
    @validate_call
    async def payment_request(
        self,
        invoice_identifier: str | UUID4,
        gateway_title: str,
        description: str | None = None,
    ):
        """Pay an invoice.

        Args:
            invoice_identifier (str): The `identifier` of the invoice.
            gateway_title (str): The gateway `title` used for the payment.

            Optionals:
                description (str): The description of the payment.

        Return:
            Essentialy the `payment_url` from the response from the server.
        """
        # Assign attributes to the request message
        PaymentRequestMessages.REQUEST.invoice = invoice_identifier
        PaymentRequestMessages.REQUEST.gateway = gateway_title
        if description:
            PaymentRequestMessages.REQUEST.description = description

        # Perform the payment operation using the perform_operation method
        return await self.perform_operation(
            PaymentRPCMethodNames.REQUEST,  # The RPC method name
            PaymentRequestMessages.REQUEST,  # The request message
            PaymentControllerStubs.INVOICE,  # The controller stub
        )

    @validate_call
    async def verify_payment(self, gateway_status: str, payment_identifier: str):
        """Verify a payment.

        Args:
            gateway_status (str): The status payment gateway claims for the payment.
            payment_identifier (str): The tracking_code of the payment.
        """
        # Assign attributes to the request message
        PaymentRequestMessages.VERIFY.gateway_status = gateway_status
        PaymentRequestMessages.VERIFY.payment_identifier = payment_identifier

        # Perform the payment operation using the perform_operation method
        return await self.perform_operation(
            PaymentRPCMethodNames.VERIFY,  # The RPC method name
            PaymentRequestMessages.VERIFY,  # The request message
            PaymentControllerStubs.INVOICE,  # The controller stub
        )


class Payment(Base):
    inquiry_request: ClassVar = PaymentRequestMessages.INQUIRY
    inquiry_method_name: ClassVar = PaymentRPCMethodNames.INQUIRY
    inquiry_controller_stub: ClassVar = PaymentControllerStubs.INQUIRY

    @validate_call
    async def request(
        self,
        invoice_identifier: str | UUID4,
        gateway_title: str,
        description: str | None = None,
    ):
        """Send a payemnt request.

        Args:
            invoice_identifier (str): The `identifier` of the invoice to pay.
            gateway_title (str): The gateway `title` used for the payment.

            Optionals:
                description (str): The description of the payment.

        Return:
            Essentialy the `payment_url` from the response from the server.
        """
        # Assign attributes to the request message
        PaymentRequestMessages.REQUEST.invoice = invoice_identifier
        PaymentRequestMessages.REQUEST.gateway = gateway_title
        if description:
            PaymentRequestMessages.REQUEST.description = description

        # Perform the payment operation using the perform_operation method
        return await self.perform_operation(
            PaymentRPCMethodNames.REQUEST,  # The RPC method name
            PaymentRequestMessages.REQUEST,  # The request message
            PaymentControllerStubs.INVOICE,  # The controller stub
        )

    @validate_call
    async def verify(self, gateway_status: str, payment_identifier: str):
        """Verify a payment.

        Args:
            gateway_status (str): The status payment gateway claims for the payment.
            payment_identifier (str): The tracking_code of the payment.
        """
        # Assign attributes to the request message
        PaymentRequestMessages.VERIFY.gateway_status = gateway_status
        PaymentRequestMessages.VERIFY.payment_identifier = payment_identifier

        # Perform the payment operation using the perform_operation method
        return await self.perform_operation(
            PaymentRPCMethodNames.VERIFY,  # The RPC method name
            PaymentRequestMessages.VERIFY,  # The request message
            PaymentControllerStubs.INVOICE,  # The controller stub
        )
